@@ -22,12 +22,7 @@
  *
  */
 
-#include "config.h"
-
-/* hack - CONFIG_SND_HDA_INPUT_JACK can be wrongly set for older kernels */
-#ifndef CONFIG_SND_JACK
-#undef CONFIG_SND_HDA_INPUT_JACK
-#endif
+#include "config-top.h"
 
 /* number of supported soundcards */
 #ifdef CONFIG_SND_DYNAMIC_MINORS
@@ -37,7 +32,9 @@
 #endif
 
 #include <linux/version.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
+#include <generated/autoconf.h>
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
 #include <linux/autoconf.h>
 #endif
 
@@ -59,6 +56,10 @@
 #endif
 #endif /* ALSA_BUILD && KERNEL < 2.6.0 */
 
+#ifndef RHEL_RELEASE_VERSION
+#define RHEL_RELEASE_VERSION(a, b) (((a) << 8) | (b))
+#endif
+
 #include <linux/module.h>
 
 #ifdef CONFIG_HAVE_OLD_REQUEST_MODULE
@@ -103,8 +104,10 @@ typedef __u32 __be32;
 
 /* other missing types */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 28)
+#if !defined(RHEL_RELEASE_CODE) || RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(5, 4)
 typedef unsigned int fmode_t;
 #endif
+#endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 /* for compat layer */
@@ -546,115 +549,8 @@ static inline void devfs_remove(const char *fmt, ...) { }
 #endif /* CONFIG_DEVFS_FS */
 #endif /* < 2.6.0 */
 
-/* workarounds for USB API */
 #if defined(SND_NEED_USB_WRAPPER) && (defined(CONFIG_USB) || defined(CONFIG_USB_MODULE))
-
-#include <linux/usb.h>
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 20)
-inline static int usb_make_path(struct usb_device *dev, char *buf, size_t size)
-{
-	int actual;
-	actual = snprintf(buf, size, "%03d/%03d", dev->bus->busnum, dev->devnum);
-	return (actual >= (int)size) ? -1 : actual;
-}
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-inline static struct urb *usb_alloc_urb_wrapper(int iso_packets, int flags)
-{
-	return usb_alloc_urb(iso_packets);
-}
-inline static int usb_submit_urb_wrapper(struct urb *urb, int flags)
-{
-	return usb_submit_urb(urb);
-}
-#undef usb_alloc_urb
-#undef usb_submit_urb
-#define usb_alloc_urb(n,flags) usb_alloc_urb_wrapper(n,flags)
-#define usb_submit_urb(p,flags) usb_submit_urb_wrapper(p,flags)
-#define OLD_USB
-#endif /* LINUX_VERSION_CODE < 2.5.0 */
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 24)
-int snd_hack_usb_set_interface(struct usb_device *dev, int interface, int alternate);
-#undef usb_set_interface
-#define usb_set_interface(dev,iface,alt) snd_hack_usb_set_interface(dev,iface,alt)
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 45)
-#define URB_ISO_ASAP		USB_ISO_ASAP
-#define URB_ASYNC_UNLINK	USB_ASYNC_UNLINK
-#define usb_fill_int_urb	FILL_INT_URB
-#define usb_fill_bulk_urb	FILL_BULK_URB
-#define usb_fill_control_urb	FILL_CONTROL_URB
-#define usb_host_config		usb_config_descriptor
-#define usb_host_interface	usb_interface_descriptor
-#define usb_host_endpoint	usb_endpoint_descriptor
-#define get_iface_desc(iface)	(iface)
-#define get_endpoint(alt,ep)	(&(alt)->endpoint[ep])
-#define get_ep_desc(ep)		(ep)
-#define get_cfg_desc(cfg)	(cfg)
-#define USB_DT_CS_DEVICE	0x21
-#define USB_DT_CS_CONFIG	0x22
-#define USB_DT_CS_STRING	0x23
-#define USB_DT_CS_INTERFACE	0x24
-#define USB_DT_CS_ENDPOINT	0x25
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 45)
-#define usb_pipe_needs_resubmit(pipe) (!usb_pipeint(pipe))
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 3, 0)
-enum {
-	USB_SPEED_UNKNOWN = 0,
-	USB_SPEED_LOW, USB_SPEED_FULL,
-	USB_SPEED_HIGH
-};
-#define snd_usb_get_speed(dev) USB_SPEED_FULL
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 8)
-#define usb_kill_urb(urb) usb_unlink_urb(urb)
-#endif
-
-#ifndef HAVE_USB_BUFFERS /* defined in usb.h */
-#define usb_buffer_alloc(dev, size, flags, dma) kmalloc(size, flags)
-#define usb_buffer_free(dev, size, addr, dma) kfree(addr)
-#undef URB_NO_TRANSFER_DMA_MAP
-#define URB_NO_TRANSFER_DMA_MAP 0
-#endif
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 3, 0) \
-    && LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 20)
-struct usb_ctrlrequest {
-	__u8 bRequestType;
-	__u8 bRequest;
-	__u16 wValue;
-	__u16 wIndex;
-	__u16 wLength;
-} __attribute__ ((packed));
-#endif
-
-#endif /* SND_NEED_USB_WRAPPER && CONFIG_USB */
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 24) \
-    && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 5) \
-    && defined(SND_NEED_USB_SET_INTERFACE) \
-    && (defined(CONFIG_USB) || defined(CONFIG_USB_MODULE))
-
-#include <linux/usb.h>
-
-inline static int real_usb_set_interface(struct usb_device *dev, int interface, int alternate)
-{
-	return usb_set_interface(dev, interface, alternate);
-}
-
-int snd_hack_usb_set_interface(struct usb_device *dev, int interface, int alternate);
-#undef usb_set_interface
-#define usb_set_interface(dev,iface,alt) snd_hack_usb_set_interface(dev,iface,alt)
-
+#include "usb_wrapper.h"
 #endif
 
 /* workqueue-alike; 2.5.45 */
@@ -728,10 +624,14 @@ static inline void snd_INIT_WORK(struct work_struct *w, void (*f)(struct work_st
 }
 #undef INIT_WORK
 #define INIT_WORK(w,f) snd_INIT_WORK(w,f)
+#ifndef SND_WORKQUEUE_COMPAT
+#define delayed_work snd_delayed_work
+#endif
 /* delayed_work wrapper */
 struct delayed_work {
 	struct work_struct work;
 };
+#undef INIT_DELAYED_WORK
 #define INIT_DELAYED_WORK(_work, _func)	INIT_WORK(&(_work)->work, _func)
 #ifndef SND_WORKQUEUE_COMPAT
 /* redefine *_delayed_work() */
@@ -924,6 +824,10 @@ static inline unsigned long msecs_to_jiffies(const unsigned int m)
 #ifndef DMA_BIT_MASK
 #define DMA_BIT_MASK(n)	(((n) == 64) ? ~0ULL : ((1ULL<<(n))-1))
 #endif
+#ifndef __GFP_ZERO
+#define __GFP_ZERO	0x4000
+#define CONFIG_HAVE_OWN_GFP_ZERO 1
+#endif
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
@@ -1409,6 +1313,7 @@ static inline int snd_pci_enable_msi(struct pci_dev *dev) { return -1; }
 /* kmemdup() wrapper */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19) || defined(CONFIG_SND_DEBUG_MEMORY)
 #include <linux/string.h>
+#include <linux/slab.h>
 static inline void *snd_kmemdup(const void *src, size_t len, gfp_t gfp)
 {
 	void *dst = kmalloc(len, gfp);
@@ -1621,15 +1526,25 @@ static inline u64 get_unaligned_be64(void *p)
 #define page_to_pfn(page)       (page_to_phys(page) >> PAGE_SHIFT)
 #endif
 
+/* strto*() wrappers */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 25)
 /* lazy wrapper - always returns 0 */
-static inline int __strict_strtoul(const char *cp, unsigned int base,
-				 unsigned long *valp)
-{
-	*valp = simple_strtoul(cp, NULL, base);
-	return 0;
+#define XXX_DEFINE_STRTO(name, type) \
+static inline int __strict_strto##name(const char *cp, unsigned int base, \
+				       type *valp) \
+{ \
+	*valp = simple_strto##name(cp, NULL, base); \
+	return 0; \
 }
-#define strict_strtoul __strict_strtoul
+XXX_DEFINE_STRTO(l, long);
+XXX_DEFINE_STRTO(ul, unsigned long);
+XXX_DEFINE_STRTO(ll, long long);
+XXX_DEFINE_STRTO(ull, unsigned long long);
+#undef XXX_DEFINE_STRTO
+#define strict_strtol	__strict_strtol
+#define strict_strtoll	__strict_strtoll
+#define strict_strtoul	__strict_strtoul
+#define strict_strtoull	__strict_strtoull
 #endif /* < 2.6.25 */
 
 /* pr_xxx() macros */
@@ -1653,6 +1568,10 @@ static inline int __strict_strtoul(const char *cp, unsigned int base,
 #define pr_warning(fmt, arg...) \
 	printk(KERN_WARNING fmt, ##arg)
 #endif
+#ifndef pr_warn
+#define pr_warn(fmt, arg...) \
+	printk(KERN_WARNING fmt, ##arg)
+#endif
 #ifndef pr_notice
 #define pr_notice(fmt, arg...) \
 	printk(KERN_NOTICE fmt, ##arg)
@@ -1686,6 +1605,8 @@ static inline const char *dev_name(struct device *dev)
 /* force to redefine WARN_ON() */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 #undef WARN_ON
+#undef WARN_ON_ONCE
+#undef WARN_ONCE
 #endif
 
 #ifndef WARN_ON
@@ -1712,6 +1633,18 @@ static inline const char *dev_name(struct device *dev)
 })
 #endif
 
+#ifndef WARN_ONCE
+#define WARN_ONCE(condition, format...)	({			\
+	static int __warned;					\
+	int __ret_warn_once = !!(condition);			\
+								\
+	if (unlikely(__ret_warn_once))				\
+		if (WARN(!__warned, format)) 			\
+			__warned = 1;				\
+	unlikely(__ret_warn_once);				\
+})
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 #define dev_printk(level, dev, format, arg...)	\
 	printk(level format, ##arg)
@@ -1785,7 +1718,9 @@ typedef int _Bool;
 #endif
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 #ifndef bool	/* just to be sure */
+#if !defined(RHEL_RELEASE_CODE) || RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(5, 4)
 typedef _Bool bool;
+#endif
 #define true	1
 #define false	0
 #endif
@@ -1820,18 +1755,8 @@ static inline void *snd_memdup_user(const void __user *src, size_t len)
 #endif
 #endif /* < 2.6.20 */
 
-/* strict_strtoull() */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 25)
-static inline int snd_strict_strtoull(const char *str, unsigned int base,
-				      unsigned long long *val)
-{
-	*val = simple_strtoull(str, NULL, base);
-	return 0;
-}
-#define strict_strtoull	snd_strict_strtoull
-#endif /* < 2.6.25 */
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 17)
+/* put_pid() function was not exported before 2.6.19 */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
 #define CONFIG_SND_HAS_REFCOUNTED_STRUCT_PID
 #endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
@@ -1885,4 +1810,182 @@ static inline pid_t pid_vnr(struct pid *pid)
 )
 #endif
 
+/* skip_spaces() wrapper */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
+char *compat_skip_spaces(const char *);
+#define skip_spaces	compat_skip_spaces
+#endif
+
+/* subsys_initcall() wrapper */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+#include <linux/init.h>
+#ifndef subsys_initcall
+#define subsys_initcall(x) module_init(x)
+#endif
+#endif
+
+/* DEFINE_PCI_DEVICE_TABLE() */
+#ifdef CONFIG_PCI
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 25)
+#ifndef DEFINE_PCI_DEVICE_TABLE
+/* originally it's __devinitconst but we use __devinitdata to be compatible
+ * with older kernels
+ */
+#define DEFINE_PCI_DEVICE_TABLE(_table) \
+	const struct pci_device_id _table[] __devinitdata
+#endif
+#endif /* < 2.6.25 */
+#endif /* CONFIG_PCI */
+
+/* blocking_notifier */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 17)
+#include <linux/notifier.h>
+#ifndef BLOCKING_INIT_NOTIFIER_HEAD
+struct blocking_notifier_head {
+	struct rw_semaphore rwsem;
+	struct notifier_block *head;
+};
+
+#define BLOCKING_INIT_NOTIFIER_HEAD(name) do {	\
+		init_rwsem(&(name)->rwsem);	\
+		(name)->head = NULL;		\
+	} while (0)
+
+static inline int
+blocking_notifier_call_chain(struct blocking_notifier_head *nh,
+			     unsigned long val, void *v)
+{
+	int ret;
+	down_read(&nh->rwsem);
+	ret = notifier_call_chain(&nh->head, val, v);
+	up_read(&nh->rwsem);
+	return ret;
+}
+
+static inline int
+blocking_notifier_chain_register(struct blocking_notifier_head *nh,
+				 struct notifier_block *nb)
+{
+	int ret;
+	down_write(&nh->rwsem);
+	ret = notifier_chain_register(&nh->head, nb);
+	up_write(&nh->rwsem);
+	return ret;
+}
+
+static inline int
+blocking_notifier_chain_unregister(struct blocking_notifier_head *nh,
+				   struct notifier_block *nb)
+{
+	int ret;
+	down_write(&nh->rwsem);
+	ret = notifier_chain_unregister(&nh->head, nb);
+	up_write(&nh->rwsem);
+	return ret;
+}
+#endif /* BLOCKING_INIT_NOTIFIER_HEAD */
+#endif /* <2.6.17 */
+
+/* pgprot_noncached - 2.4 has different defines */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
+#ifndef pgprot_noncached
+#define pgprot_noncached(x) (x)
+#endif
+#endif
+
+/* no_llseek() */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 0)
+#define no_llseek	NULL
+#endif
+
+/* noop_llseek() */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)
+#define noop_llseek	NULL
+#endif
+
+/* nonseekable_open() */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 8)
+#define nonseekable_open(i,f) 0
+#endif
+
+/* hex_to_bin() */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)
+static inline int hex_to_bin(char c)
+{
+	if (c >= '0' && c <= '9')
+		return c - '0';
+	if (c >= 'A' && c <= 'F')
+		return c - 'A' + 10;
+	if (c >= 'a' && c <= 'f')
+		return c - 'a' + 10;
+	return -1;
+}
+#endif
+
+/* vzalloc() wrapper */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 37)
+#include <linux/vmalloc.h>
+static inline void *vzalloc(unsigned long size)
+{
+	void *p = vmalloc(size);
+	if (p)
+		memset(p, 0, size);
+	return p;
+}
+#endif
+
+/* flush_delayed_work_sync() wrapper */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 37)
+#include <linux/workqueue.h>
+static inline bool flush_work_sync(struct work_struct *work)
+{
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 27)
+	/* XXX */
+	flush_scheduled_work();
+	return true;
+#else
+	if (!flush_work(work))
+		return false;
+	while (flush_work(work))
+		;
+	return true;
+#endif
+}
+
+static inline bool flush_delayed_work_sync(struct delayed_work *dwork)
+{
+	bool ret;
+	ret = cancel_delayed_work(dwork);
+	if (ret) {
+		schedule_delayed_work(dwork, 0);
+		flush_scheduled_work();
+	}
+	return ret;
+}
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
+/* XXX this is a workaround; these are really different, but almost same
+ * as used in the usual free/error path
+ */
+#define cancel_delayed_work_sync flush_delayed_work_sync
+#endif
+
+#endif /* < 2.6.37 */
+
+/* pm_wakeup_event() wrapper */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 37)
+#define pm_wakeup_event(dev, msec)
+#endif
+
+/* request_any_context_irq() wrapper */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)
+#define request_any_context_irq(irq, fn, flags, name, dev_id) \
+	request_irq(irq, fn, flags, name, dev_id)
+#endif
+
+/* hack - CONFIG_SND_HDA_INPUT_JACK can be wrongly set for older kernels */
+#ifndef CONFIG_SND_JACK
+#undef CONFIG_SND_HDA_INPUT_JACK
+#endif
+
 #endif /* __SOUND_LOCAL_DRIVER_H */
