@@ -165,12 +165,19 @@ static char *kernel_deps[] = {
 	"HAS_IOPORT",
 	"EXPERIMENTAL",
 	"BROKEN",
+	"LZO_COMPRESS",
+	"LZO_DECOMPRESS",
 	/* sound common */
 	"AC97_BUS",
 	/* workaround */
 	"SND_SOC_ALL_CODECS",
 	/* this is used only in i2c/cs8427.c, and we have a wrapper there */
 	"%BITREVERSE",
+	/* used in core/pcm_timer.c, a wrapper present */
+	"%GCD",
+	/* others */
+	"%HAS_IOMEM",
+	"!M68K",
 	NULL
 };
 
@@ -179,10 +186,8 @@ static char *kernel_deps[] = {
 /* # -> menuconfig */
 static char *no_cards[] = {
 	"%SOUND",
-	"%HAS_IOMEM",
 	"SOUND_PRIME",
 	"%SND",
-	"!M68K",
 	/* options given as configure options */
 	"SND_DYNAMIC_MINORS",
 	"SND_DEBUG",
@@ -219,6 +224,7 @@ static char *no_cards[] = {
 #define READ_STATE_CONFIG	1
 #define READ_STATE_MENU		2
 #define READ_STATE_COMMENT	3
+#define READ_STATE_CHOICE	4
 
 static void nomem(void)
 {
@@ -393,6 +399,13 @@ static int read_file_1(const char *filename, struct cond **template)
 		} else if (!strncmp(buffer, "comment", 7)) {
 			state = READ_STATE_COMMENT;
 			continue;
+		} else if (!strncmp(buffer, "choice", 6)) {
+			state = READ_STATE_CHOICE;
+			/* FXIME: what to do? */
+			continue;
+		} else if (!strncmp(buffer, "endchoice", 9)) {
+			state = READ_STATE_NONE;
+			continue;
 		}
 		switch (state) {
 		case READ_STATE_CONFIG:
@@ -989,13 +1002,13 @@ static int is_toplevel(struct dep *dep)
 	return 1;
 }
 
-static int check_in_no_cards(struct dep *dep, char flag)
+static int check_in_list(struct dep *dep, char flag, char **list)
 {
 	char **p;
 
 	if (!dep)
 		return 0;
-	for (p = no_cards; *p; p++) {
+	for (p = list; *p; p++) {
 		if (**p != flag)
 			continue;
 		if (!strcmp(*p + 1, dep->name))
@@ -1003,6 +1016,13 @@ static int check_in_no_cards(struct dep *dep, char flag)
 	}
 	return 0;
 }
+
+static int check_in_no_cards(struct dep *dep, char flag)
+{
+	return check_in_list(dep, flag, no_cards) ||
+		check_in_list(dep, flag, kernel_deps);
+}
+
 // is CONFIG_ variable is always true
 static int is_always_true(struct dep *dep)
 {
@@ -1494,6 +1514,8 @@ static void output_include(void)
 	printf("/* Copyright (c) by Jaroslav Kysela <perex@perex.cz>, */\n");
 	printf("/*                  Anders Semb Hermansen <ahermans@vf.telia.no> */\n\n");
 	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
+		if (is_kernel_deps(tempdep->name))
+			continue;
 		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
 		printf("#undef %s%s\n", text,
 		       tempdep->type == TYPE_TRISTATE ? "_MODULE" : "");
